import { NextResponse } from "next/server"
import { z } from "zod"

import { db } from "@/lib/db"

/**
 * @swagger
 * /api/hashtags:
 *   get:
 *     description: Gives back a List of Hashtags
 *     content:
 *      application/json:
 *          schema:
 *     responses:
 *       200:
 *         description: fetched hashtags!
 *       500:
 *         description: Error
 *
 *   post:
 *     description: Creates Hashtag
 *     responses:
 *       200:
 *         description: Hashtag created!
 *       401:
 *         description: Unauthorized
 *       500:
 *         description: Error
 *
 */

export async function GET() {
    try {
        const hashtags = await db.hashtag.findMany({
            take: 10,
            orderBy: {
                score: "desc",
            },
        })
        return NextResponse.json(hashtags, { status: 200 })
    } catch (error: any) {
        return NextResponse.json({ error: error.message }, { status: 500 })
    }
}

export async function POST(request: Request) {
    const hashtags = await request.json()

    const hashtagsSchema = z.array(z.string())

    const zod = hashtagsSchema.safeParse(hashtags)

    if (!zod.success) {
        return NextResponse.json({ error: zod.error }, { status: 400 })
    }

    try {
        for (const hashtag of hashtags) {
            const hashtagExists = await db.hashtag.findUnique({
                where: {
                    hashtag: hashtag.toLowerCase(),
                },
            })

            if (hashtagExists) {
                await db.hashtag.update({
                    where: {
                        hashtag: hashtag.toLowerCase(),
                    },
                    data: {
                        score: {
                            increment: 1,
                        },
                    },
                })
            } else {
                await db.hashtag.create({
                    data: {
                        text: hashtag,
                        hashtag: hashtag.toLowerCase(),
                    },
                })
            }
        }

        return NextResponse.json({ message: "Hashtag(s) created", }, { status: 201 })
    } catch (error: any) {
        return NextResponse.json({ error: error.message, }, { status: 500 },)
    }
}